@@ -46,7 +46,7 @@ class RunResult:
         stderr: Standard error from the code execution
         returncode: Exit code from the code execution (0 = success)
         code: The original code that was executed
-        language: The language that was executed (currently only python)
+        language: The language that was executed (python or javascript)
     """
 
     stdout: str
@@ -267,7 +267,7 @@ class Sandbox:
     Main interface for interacting with the Concave sandbox service.
 
     This class manages the lifecycle of isolated code execution environments,
-    providing methods to create, execute commands, run Python code, and clean up
+    providing methods to create, execute commands, run Python and JavaScript code, and clean up
     sandbox instances. Each sandbox is backed by a Firecracker VM for strong
     isolation while maintaining fast performance.
 
@@ -349,6 +349,9 @@ class Sandbox:
             raise SandboxAuthenticationError(f"Authentication failed: {error_msg}") from e
         elif status_code == 404:
             raise SandboxNotFoundError(f"Not found: {error_msg}") from e
+        elif status_code == 408:
+            # Request timeout from backend
+            raise SandboxTimeoutError(f"Operation timed out: {error_msg}", timeout_ms=None, operation=operation) from e
         elif status_code == 429:
             raise SandboxRateLimitError(f"Rate limit exceeded: {error_msg}") from e
         elif status_code == 500:
@@ -507,7 +510,8 @@ class Sandbox:
         cls, 
         internet_access: bool = True, 
         metadata: Optional[dict[str, str]] = None,
-        env: Optional[dict[str, str]] = None
+        env: Optional[dict[str, str]] = None,
+        lifetime: Optional[str] = None
     ) -> "Sandbox":
         """
         Create a new sandbox instance.
@@ -516,13 +520,15 @@ class Sandbox:
             internet_access: Enable internet access for the sandbox (default: True)
             metadata: Optional immutable metadata to attach (key-value pairs)
             env: Optional custom environment variables to inject into the sandbox
+            lifetime: Optional sandbox lifetime (e.g., "1h", "30m", "1h30m"). 
+                     Min: 1m, Max: 48h, Default: 24h
 
         Returns:
             A new Sandbox instance ready for code execution
 
         Raises:
             SandboxCreationError: If sandbox creation fails
-            SandboxValidationError: If metadata or env validation fails
+            SandboxValidationError: If metadata, env, or lifetime validation fails
             ValueError: If CONCAVE_SANDBOX_API_KEY environment variable is not set
 
         Example:
@@ -530,6 +536,8 @@ class Sandbox:
             sbx_no_internet = Sandbox.create(internet_access=False)
             sbx_with_meta = Sandbox.create(metadata={"env": "prod", "user": "123"})
             sbx_with_env = Sandbox.create(env={"API_KEY": "secret", "DEBUG": "true"})
+            sbx_short_lived = Sandbox.create(lifetime="1h")
+            sbx_long_lived = Sandbox.create(lifetime="48h")
         """
         # Get credentials using helper method
         base_url, api_key = cls._get_credentials(None, None)
@@ -600,6 +608,23 @@ class Sandbox:
             if total_size > 4096:
                 raise SandboxValidationError(f"total env size ({total_size} bytes) exceeds limit of 4096 bytes")
 
+        # Validate lifetime if provided
+        if lifetime is not None:
+            import re
+            
+            if not isinstance(lifetime, str):
+                raise SandboxValidationError("lifetime must be a string")
+            
+            # Validate format: must match pattern like "1h", "30m", "1h30m", "90s"
+            # Pattern: optional hours, optional minutes, optional seconds (at least one required)
+            lifetime_regex = re.compile(r'^((\d+h)?(\d+m)?(\d+s)?|(\d+h\d+m\d+s)|(\d+h\d+m)|(\d+h\d+s)|(\d+m\d+s))$')
+            if not lifetime_regex.match(lifetime) or lifetime == "":
+                raise SandboxValidationError(f"lifetime '{lifetime}' has invalid format (use formats like '1h', '30m', '1h30m', '90s')")
+            
+            # Check if it contains at least one time component
+            if not any(c in lifetime for c in ['h', 'm', 's']):
+                raise SandboxValidationError(f"lifetime '{lifetime}' must contain at least one time unit (h, m, or s)")
+
         # Create HTTP client using helper method
         client = cls._create_http_client(api_key)
 
@@ -611,6 +636,8 @@ class Sandbox:
                 payload["metadata"] = metadata
             if env:
                 payload["env"] = env
+            if lifetime:
+                payload["lifetime"] = lifetime
             response = client.put(f"{base}/api/v1/sandboxes", json=payload)
             response.raise_for_status()
             sandbox_data = response.json()
@@ -1178,6 +1205,14 @@ class Sandbox:
                     error_msg = data.get("error") if isinstance(data, dict) else None
                     if isinstance(error_msg, str) and "sandbox not found" in error_msg.lower():
                         raise SandboxNotFoundError(f"Sandbox {self.id} not found")
+                    # Handle timeout responses (408 Request Timeout)
+                    if resp.status_code == 408:
+                        timeout_val = timeout if timeout else 300000
+                        raise SandboxTimeoutError(
+                            f"Command execution timed out: {error_msg if error_msg else 'timeout'}",
+                            timeout_ms=timeout_val,
+                            operation="execute_stream"
+                        )
                     raise SandboxExecutionError(
                         f"Execute streaming failed: {error_msg if error_msg else raw.decode('utf-8', errors='replace')}"
                     )
@@ -1249,7 +1284,7 @@ class Sandbox:
         Args:
             code: Code to execute
             timeout: Timeout in milliseconds (default: 10000ms)
-            language: Programming language to use (default: "python"). Currently only Python is supported.
+            language: Programming language to use (default: "python"). Supported: "python", "javascript".
 
         Returns:
             RunResult containing stdout, stderr, return code, original code, and language
@@ -1264,6 +1299,10 @@ class Sandbox:
             result = sbx.run("print('Hello, World!')")
             print(result.stdout)  # Hello, World!
             
+            # Run JavaScript code
+            result = sbx.run("console.log('Hello, World!')", language="javascript")
+            print(result.stdout)  # Hello, World!
+            
             # Run Python with timeout
             result = sbx.run("import time; time.sleep(1)", timeout=3000)
             print(result.stdout)
@@ -1271,8 +1310,8 @@ class Sandbox:
         if not code.strip():
             raise SandboxValidationError("Code cannot be empty")
 
-        if language != "python":
-            raise SandboxValidationError(f"Unsupported language: {language}. Currently only 'python' is supported.")
+        if language not in ("python", "javascript"):
+            raise SandboxValidationError(f"Unsupported language: {language}. Currently only 'python' and 'javascript' are supported.")
 
         # Non-streaming path (JSON response)
         if not streaming:
@@ -1359,6 +1398,14 @@ class Sandbox:
                     error_msg = data.get("error") if isinstance(data, dict) else None
                     if isinstance(error_msg, str) and "sandbox not found" in error_msg.lower():
                         raise SandboxNotFoundError(f"Sandbox {self.id} not found")
+                    # Handle timeout responses (408 Request Timeout)
+                    if resp.status_code == 408:
+                        timeout_val = timeout if timeout else 300000
+                        raise SandboxTimeoutError(
+                            f"Code execution timed out: {error_msg if error_msg else 'timeout'}",
+                            timeout_ms=timeout_val,
+                            operation="run_stream"
+                        )
                     raise SandboxExecutionError(
                         f"Run streaming failed: {error_msg if error_msg else raw.decode('utf-8', errors='replace')}"
                     )
@@ -1681,6 +1728,129 @@ class Sandbox:
         except httpx.RequestError as e:
             raise SandboxConnectionError(f"Failed to connect to sandbox service: {e}") from e
 
+    def publish(self, port: int) -> str:
+        """
+        Expose a sandbox port via a public *.concave.run URL.
+        
+        Args:
+            port: Internal sandbox port to expose (e.g., 3000, 8000, 8080)
+        
+        Returns:
+            str: Public URL to access the exposed port (e.g., "a1b2c3d4.concave.run")
+        
+        Raises:
+            SandboxNotFoundError: If the sandbox doesn't exist
+            SandboxValidationError: If port is invalid (not 1-65535)
+            SandboxAuthenticationError: If API authentication fails
+            SandboxServerError: If the expose service fails
+            SandboxConnectionError: If unable to connect to the service
+            SandboxTimeoutError: If the request times out
+        
+        Example:
+            sbx = Sandbox.create()
+            
+            # Start a web server in the sandbox
+            sbx.execute("nohup python3 -m http.server 8000 > /tmp/server.log 2>&1 &")
+            
+            # Expose it publicly
+            url = sbx.publish(8000)
+            print(f"Access your server at: https://{url}")
+            # Output: Access your server at: https://a1b2c3d4.concave.run
+            
+            # Now anyone can access https://a1b2c3d4.concave.run
+        
+        Note:
+            When a sandbox is deleted, all published ports are automatically
+            unpublished and their *.concave.run URLs become invalid.
+        """
+        if not isinstance(port, int) or port < 1 or port > 65535:
+            raise SandboxValidationError(f"Port must be an integer between 1 and 65535, got: {port}")
+        
+        try:
+            response = self._client.post(
+                f"{self._sandboxes_url}/{self.id}/publish",
+                json={"port": port},
+                timeout=10.0,
+            )
+            response.raise_for_status()
+            data = response.json()
+            
+            # Extract URL from response
+            if "url" not in data:
+                raise SandboxInvalidResponseError(
+                    f"Invalid publish response: missing 'url' field"
+                )
+            
+            return data["url"]
+        
+        except httpx.HTTPStatusError as e:
+            self._handle_http_error(e, "publish port")
+        except httpx.TimeoutException as e:
+            raise SandboxTimeoutError(
+                "Port publish request timed out", timeout_ms=10000, operation="publish"
+            ) from e
+        except httpx.RequestError as e:
+            raise SandboxConnectionError(f"Failed to connect to sandbox service: {e}") from e
+
+    def unpublish(self, port: int) -> bool:
+        """
+        Remove public exposure of a sandbox port.
+        
+        Args:
+            port: Internal sandbox port to unpublish
+        
+        Returns:
+            bool: True if unpublish succeeded (always returns True unless error)
+        
+        Raises:
+            SandboxNotFoundError: If the sandbox doesn't exist
+            SandboxValidationError: If port is invalid
+            SandboxAuthenticationError: If API authentication fails
+            SandboxServerError: If the expose service fails
+            SandboxConnectionError: If unable to connect to the service
+            SandboxTimeoutError: If the request times out
+        
+        Note:
+            Returns True even if the port was not published. Only returns False
+            or raises exception on actual errors.
+        
+        Example:
+            sbx = Sandbox.create()
+            url = sbx.publish(8000)
+            print(f"Published: {url}")
+            
+            # Later, remove the exposure
+            success = sbx.unpublish(8000)
+            print(f"Unpublished: {success}")  # True
+            
+            # Unpublishing again still returns True
+            success = sbx.unpublish(8000)  # Still True
+        """
+        if not isinstance(port, int) or port < 1 or port > 65535:
+            raise SandboxValidationError(f"Port must be an integer between 1 and 65535, got: {port}")
+        
+        try:
+            response = self._client.request(
+                "DELETE",
+                f"{self._sandboxes_url}/{self.id}/unpublish",
+                json={"port": port},
+                timeout=10.0,
+            )
+            response.raise_for_status()
+            data = response.json()
+            
+            # Check success field
+            return data.get("success", True)
+        
+        except httpx.HTTPStatusError as e:
+            self._handle_http_error(e, "unpublish port")
+        except httpx.TimeoutException as e:
+            raise SandboxTimeoutError(
+                "Port unpublish request timed out", timeout_ms=10000, operation="unpublish"
+            ) from e
+        except httpx.RequestError as e:
+            raise SandboxConnectionError(f"Failed to connect to sandbox service: {e}") from e
+
     @property
     def info(self) -> Dict[str, Any]:
         """
@@ -2210,7 +2380,8 @@ class Sandbox:
 def sandbox(
     internet_access: bool = True, 
     metadata: Optional[dict[str, str]] = None,
-    env: Optional[dict[str, str]] = None
+    env: Optional[dict[str, str]] = None,
+    lifetime: Optional[str] = None
 ):
     """
     Context manager for creating and automatically cleaning up a sandbox.
@@ -2222,13 +2393,15 @@ def sandbox(
         internet_access: Enable internet access for the sandbox (default: True)
         metadata: Optional immutable metadata to attach (key-value pairs)
         env: Optional custom environment variables to inject into the sandbox
+        lifetime: Optional sandbox lifetime (e.g., "1h", "30m", "1h30m"). 
+                 Min: 1m, Max: 48h, Default: 24h
 
     Yields:
         Sandbox: A sandbox instance ready for code execution
 
     Raises:
         SandboxCreationError: If sandbox creation fails
-        SandboxValidationError: If metadata or env validation fails
+        SandboxValidationError: If metadata, env, or lifetime validation fails
         ValueError: If CONCAVE_SANDBOX_API_KEY environment variable is not set
 
     Example:
@@ -2254,9 +2427,14 @@ def sandbox(
         with sandbox(env={"API_KEY": "secret", "DEBUG": "true"}) as s:
             result = s.run("import os; print(os.environ['API_KEY'])")
             print(result.stdout)
+        
+        # Create sandbox with custom lifetime
+        with sandbox(lifetime="5h") as s:
+            result = s.run("print('This sandbox lives for 5 hours!')")
+            print(result.stdout)
         ```
     """
-    sbx = Sandbox.create(internet_access=internet_access, metadata=metadata, env=env)
+    sbx = Sandbox.create(internet_access=internet_access, metadata=metadata, env=env, lifetime=lifetime)
     try:
         yield sbx
     finally:
